#!/usr/bin/env python3
"""
Copyright 2022 Byron Labs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import json

from pymisp import MISPAttribute, MISPEvent, MISPObject, MISPTag

try:
    from types import NoneType
except:
    NoneType = type(None)

import vysion.dto as dto
from vysion.dto import URL, Hit, Page, RansomFeedHit


class MISPProcessor:

    def __init__(self):
        self.misp_event = MISPEvent()

    def parse_hit(self, hit: Hit, ref_attribute: MISPAttribute = None, **_):

        page: Page = hit.page

        # TODO Add more page parameters
        misp_object = MISPObject("vysion-page")
        misp_object.template_uuid = "4d0b66f1-5268-47e0-9d29-f2e4f3db8e7f"

        page_id = page.id
        misp_object.add_attribute("id", type="text", value=page_id)

        url: URL = page.url
        misp_object.add_attribute("url", type="url", value=url.build())

        network = url.network
        misp_object.add_attribute("network", type="text", value=network)

        title = page.title
        misp_object.add_attribute("title", type="text", value=title)

        if ref_attribute is not None:
            misp_object.add_reference(ref_attribute.uuid, "associated-to")
  
        self.misp_event.add_object(misp_object)

        vysion_reference_id = misp_object.uuid

        # TODO Remove this addition when the vysion-page object works
        self.misp_event.add_attribute("url", value=url.build())

        self.misp_event.add_attribute("domain", value=url.domain)

        for email in hit.email:
            self.misp_event.add_attribute("email", value=email.value)

        for btc in hit.bitcoin_address:
            self.misp_event.add_attribute("btc", value=btc.value)

        for dot in hit.polkadot_address:
            self.misp_event.add_attribute("dot", value=dot.value)

        for eth in hit.ethereum_address:
            self.misp_event.add_attribute("eth", value=eth.value)

        for xmr in hit.monero_address:
            self.misp_event.add_attribute("xmr", value=xmr.value)

        for xrp in hit.ripple_address:
            self.misp_event.add_attribute("xrp", value=xmr.value)

        for zec in hit.zcash_address:
            self.misp_event.add_attribute("zec", value=zec.value)

        for tag in hit.tag:
            self.misp_event.add_tag(str(tag))

    def parse_ransom_feed_hit(self, hit: RansomFeedHit, **kwargs):

        # TODO Add event info!

        misp_object = MISPObject("vysion-ransomware-feed")
        misp_object.template_uuid = "e0bfa994-c184-4894-bfaa-73b1350746e1"
        misp_object["meta-category"] = "misc" # TODO Esto se tiene que poder hacer de otra manera... Y sólo es necesario en los feeds

        misp_object.add_attribute("id", type="text", value=hit.id)
        misp_object.add_attribute("company", type="target-org", value=hit.company)
        misp_object.add_attribute("company_link", type="link", value=hit.company_link)
        misp_object.add_attribute("link", type="link", value=hit.link)
        misp_object.add_attribute("group", type="threat-actor", value=hit.group)
        misp_object.add_attribute("date", type="datetime", value=hit.date)
        misp_object.add_attribute("info", type="text", value=hit.info)
        misp_object.add_attribute("country", type="text", value=hit.country)

        self.misp_event.add_object(misp_object)

    def process(self, result: dto.Result, **kwargs) -> MISPEvent:

        processor = {
            Hit: self.parse_hit,
            RansomFeedHit: self.parse_ransom_feed_hit,
        }.get(result.get_type(), lambda *_, **__: {})

        for hit in result.hits:
            processor(hit, **kwargs)

        return self.misp_event



#!/usr/bin/env python3
"""
Copyright 2022 Byron Labs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# TODO Referenciar vt-py

import logging
from datetime import datetime, timedelta
from typing import Union
from urllib.parse import urlencode, urljoin

# from pydantic import validate_arguments
import requests

import vysion.dto as dto
from vysion.client.error import APIError
from vysion.dto import VysionError
from vysion.version import __version__ as vysion_version

_API_HOST = "https://api.vysion.ai"

# All API endpoints start with this prefix, you don't need to include the
# prefix in the paths you request as it's prepended automatically.
_ENDPOINT_PREFIX = "/api/v1/"

_BASE_API = urljoin(_API_HOST, _ENDPOINT_PREFIX)

LOGGER = logging.getLogger("vysion-py")
LOGGER.setLevel(logging.INFO)

# __all__ = []


class BaseClient:

    # __attrs__ = []

    # @validate_arguments #
    def __init__(self, api_key: str = None, headers: dict = dict(), proxy: dict = None):

        assert api_key is not None, "API key MUST be provided"
        assert isinstance(api_key, str), "API key MUST be a string"

        self.api_key = api_key
        self.proxy = proxy  # TODO Rename proxy to proxies
        self.headers = headers

    def __get_session__(self) -> requests.Session:

        # TODO Configure proxy

        # If session is undefined
        try:
            self._session
        except (NameError, AttributeError):

            headers = self.headers.copy()
            headers.update(
                {
                    "X-API-KEY": self.api_key,
                    "User-Agent": "vysion-py/%s" % vysion_version,
                }
            )

            self._session: requests.Session = requests.Session()
            self._session.headers.update(headers)
            self._session.proxies = self.proxy

        return self._session

    def _build_api_url__(self, endpoint, param, **query_params):

        base = urljoin(_BASE_API, f"{endpoint}/{param}")

        query_params_initialzed = query_params.copy()

        keys = list(query_params.keys())
        keys.sort()

        for i in keys:

            v = query_params[i]

            if v is None:
                del query_params_initialzed[i]

        query = "?" + urlencode(query_params_initialzed)

        return urljoin(base, query)

    def _make_request(self, url: str) -> dto.VysionResponse:

        session = self.__get_session__()
        r = session.get(url)

        # TODO Improve this
        if r.status_code != 200:
            try:
                err = r.json()
                code = err.get("code")
                message = err.get("message")
            except:
                code = r.status_code
                message = r.text

            raise APIError(code, message)

        payload = r.json()

        result = dto.VysionResponse.parse_obj(payload)

        return result


def vysion_error_manager(method) -> Union[dto.Result, VysionError]:

    def manage(*args, **kwargs):
        try:
            result = method(*args, **kwargs)
            return result
        except APIError as e:
            return VysionError(code=e.code, message=e.message)
        except Exception as e:
            LOGGER.error(e)
            return VysionError()

    return manage


class Client(BaseClient):

    # def add_url(self, url:str, type:VysionURL.Type):
    #     """Add a Tor URL to be analyzed by PARCHE.

    #     :param url: URL to be scanned.
    #     :param type: Instance of :class:`VysionURL.Type`
    #     :returns: An instance of :class:`VysionResponse`
    #     """
    #     pass

    # def find_onion(self):
    #   pass

    # def add_onion(self):
    #   pass

    # def consume_feed(self):
    #   pass

    @vysion_error_manager
    def status(self):
        # TODO Check API status
        pass

    @vysion_error_manager
    def search(
        self,
        query: str,
        tag: str = None,
        notTag: str = None,
        exact: bool = False,
        network: dto.Network = None,
        language: dto.Language = None,
        page: int = 1,
        lte: datetime = None,
        gte: datetime = None,
    ) -> dto.Result:

        url = self._build_api_url__(
            "search",
            query,
            tag=tag,
            notTag=notTag,
            exact=exact,
            network=network,
            language=language,
            page=page,
            lte=lte,
            gte=gte,
        )

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_btc(
        self, btc: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("btc", btc, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_eth(
        self, eth: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("eth", eth, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_dot(
        self, dot: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("dot", dot, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_xrp(
        self, xrp: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("xrp", xrp, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_xmr(
        self, xmr: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("xmr", xmr, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_zec(
        self, zec: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__("zec", zec, page=page, lte=lte, gte=gte)

        result = self._make_request(url)
        return result.data

    # TODO find_domain?
    @vysion_error_manager
    def find_url(
        self,
        query_url: str,
        page: int = 1,
        lte: datetime = None,
        gte: datetime = None,
    ) -> dto.Result:

        url = self._build_api_url__(
            "url", query_url, page=page, lte=lte, gte=gte
        )

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def find_email(
        self, email: str, page: int = 1, lte: datetime = None, gte: datetime = None
    ) -> dto.Result:

        url = self._build_api_url__(
            "email", email, page=page, lte=lte, gte=gte
        )

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def get_document(self, document_id: str) -> dto.Result:

        url = self._build_api_url__("document", document_id)

        result = self._make_request(url)
        return result.data

    @vysion_error_manager
    def get_tag(self, tag: str) -> dto.Result:

        url = self._build_api_url__("tag", tag)

        result = self._make_request(url)
        return result.data

    # FEEDS
    @vysion_error_manager
    def consume_feed_ransomware(self, batch_day: datetime = datetime.today()):
        pass


# TODO Develop feeds logic
# Example: https://github.com/VirusTotal/vt-py/blob/master/vt/feed.py
class DaylyFeed(Client):
    def _consume_batch(self, start_time, end_time):
        raise NotImplementedError()

    def consume(self, batch_day: datetime = datetime.today()):
        start_time = datetime(datetime.year, datetime.month, datetime.day)
        end_time = start_time + timedelta(days=1)
        return self._consume_batch(start_time, end_time)


class RansomwareFeed(DaylyFeed):
    def _consume_batch(self, start_time, end_time):

        days = (datetime.now() - start_time).days
        pages = (end_time - start_time).days

        for page in range(pages):
            url = self._build_api_url__("feed", "ransomware", days=days, page=page + 1)
            yield self._make_request(url)



# TODO /api/v1/feeds



#!/usr/bin/env python3
"""
Copyright 2022 ByronLabs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import hashlib
import uuid
from datetime import datetime

from .enum import Enum

try:
    from types import NoneType
except:
    NoneType: type = type(None)

import re
from typing import List, Optional, Union

from pydantic import BaseModel, Field  # , constr

from .enum import Network, Services

NULL_UUID = uuid.UUID("00000000-0000-0000-0000-000000000000")

class URL(BaseModel):

    protocol: Optional[str] 
    domain: Optional[str]   
    port: Optional[int] = Field(default_factory=lambda: -1)
    path: Optional[str]
    signature: uuid.UUID = Field(default_factory=lambda: NULL_UUID)
    
    raw: str

    @staticmethod
    def _parse_(url):
        
        """
        RFC3986
            scheme    = $2
            authority = $4
            path      = $5
            query     = $7
            fragment  = $9
        """
        regex = r"^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?"
        
        res = re.match(regex, url)
        
        scheme = res[2]
        authority = res[4]
        path = res[5]
        query = res[7]
        fragment = res[9]

        return dict(
            scheme=scheme,
            authority=authority,
            path=path,
            query=query,
            fragment=fragment
        )

    def _generate_signature_(self) -> uuid.UUID:
        return uuid.uuid5(uuid.NAMESPACE_URL, self.build())

    @classmethod
    def parse(cls, url, fix=False):

        # Saving the original url
        raw = url

        parsed = cls._parse_(url)
        
        # Elements
        scheme = parsed["scheme"]
        netloc = parsed["authority"]
        path = parsed["path"]
        query = parsed["query"]
        fragment = parsed["fragment"]

        # Build domain:port
        if netloc is not None:
            domain_port = (netloc.split(":") + [None])[:2]
        else:
            domain_port = [None, None]
        
        domain = domain_port[0]
        port = domain_port[1]

        # Normalize path: /path?query#fragment
        # Parse a URL into 6 components:
        # <protocol>://<domain>:<port>/<path>
        if query is not None:

            # Normalize query
            query_parts = [param.split("=") for param in query.split("&")]
            query_dict = {}
            for part in query_parts:
                if len(part) <= 1:
                    query_dict[part[0]] = str()
                else:
                    query_dict[part[0]] = part[1]

            query_keys = list(query_dict.keys())
            query_keys.sort()
            res_query_parts = [f"{k}={query_dict[k]}" for k in query_keys]

            res_query = "?" + "&".join(res_query_parts)
        
            path += res_query
        
        if fragment is not None:
            path += f"#{fragment}"

        # TODO Adapt restalker.link_extractors.UUF logic to fix URLs
        # TODO Detect network?
        result = cls(
            protocol=scheme,
            domain=domain,
            port=port,
            path=path,
            raw=raw
        )

        if fix:
            result._fix()

        return result

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.signature = self._generate_signature_()

    def build(self) -> str:
        
        url = self.path

        if self.domain is not None:
            
            if self.port is not None:
                url = f":{self.port}" + url
            
            url = f"{self.domain}" + url

        if self.protocol is not None:

            url = f"{self.protocol}://" + url
            
        return url

    def _fix(self, default_protocol=Services.http) -> None:

        if self.protocol is None:
            if self.port is None:
                self.protocol = default_protocol.name
            else:
                self.protocol = Services(self.port).name

        if self.port is None:
            self.port = Services[self.protocol].value

        if self.domain is None:
            path = self.path
            if path[0] == "/":
                path = path[1:]
            path_parts = path.split('/')
            self.domain = path_parts[0]
            self.path = "/" + "/".join(path_parts[1:])
        
        self.signature = self._generate_signature_()

    def __str__(self) -> str:
        return self.build()

    def __repr__(self):
        return self.build()



#!/usr/bin/env python3
"""
Copyright 2022 ByronLabs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import hashlib
from datetime import datetime
from enum import Enum

from vysion.model import enum
from vysion.taxonomy import Monero_Address, Ripple_Address

try:
    from types import NoneType
except:
    NoneType: type = type(None)

from typing import List, Optional, Union
from urllib.parse import urlparse

from pydantic import BaseModel, Field  # , constr

from vysion import taxonomy as vystaxonomy
from vysion.model import URL as URL_model
from vysion.model.enum import Language, Network, RansomGroup, Services

from .tag import *


class Email(BaseModel):

    _taxonomy = [vystaxonomy.Email]

    # RFC 5322 Official Standard (https://www.emailregex.com/)
    # value: constr(regex=r'''(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])''') # TODO Añadir que es str
    value: str  # TODO Fix regex to allow caps


class Paste(BaseModel):

    _taxonomy = [vystaxonomy.Pastebin, vystaxonomy.JustPaste]

    value: str  # TODO Regex


class Skype(BaseModel):

    _taxonomy = [vystaxonomy.Skype]

    value: str  # TODO Regex


class Telegram(BaseModel):

    _taxonomy = [vystaxonomy.Telegram]  # TODO Create Telegram URL

    value: str  # TODO Regex


class BitcoinAddress(BaseModel):

    _taxonomy = [vystaxonomy.Bitcoin_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class PolkadotAddress(BaseModel):

    _taxonomy = [vystaxonomy.Polkadot_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class EthereumAddress(BaseModel):

    _taxonomy = [vystaxonomy.Ethereum_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class MoneroAddress(BaseModel):

    _taxonomy = [vystaxonomy.Monero_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class RippleAddress(BaseModel):

    _taxonomy = [vystaxonomy.Ripple_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class ZcashAddress(BaseModel):

    _taxonomy = [vystaxonomy.Zcash_Address]  # TODO Create Telegram URL

    value: str  # TODO Regex


class WhatsApp(BaseModel):

    _taxonomy = [vystaxonomy.WhatsApp]

    value: str  # TODO Regex


class URL(BaseModel):

    _taxonomy = [vystaxonomy.URL]

    protocol: Optional[str]
    domain: Optional[str]
    port: Optional[int]
    path: Optional[str]
    signature: str
    network: Network = Field(default_factory=lambda: Network.clearnet)

    @classmethod
    def parse(cls, url):

        # TODO Save this parsed in a private variable? (e.g., _pared_)
        parsed = URL_model.parse(url)
        print(parsed)
        tmp_result = cls(
            protocol=parsed.protocol,
            domain=parsed.domain,
            port=parsed.port,
            path=parsed.path,
            signature=str(parsed.signature) # TODO Replace signature: str --> UUID
        )

        return tmp_result

    def build(self) -> str:
        return f"{self.protocol}://{self.domain}:{self.port}{self.path}"


class Page(BaseModel):

    id: str
    url: URL
    parent: str = None
    title: str = None  # TODO Revisar si None o str()
    language: Optional[Language]
    html: str = None
    sha1sum: str = None
    sha256sum: str = None
    ssdeep: str = None
    date: datetime = None
    chunk: bool = False


class RansomwarePage(BaseModel):

    id: str
    url: URL
    # title: str = None  
    group: RansomGroup
    company: Optional[str]
    company_address: Optional[str]
    company_link: Optional[str]
    info: Optional[str]
    html: Optional[str]
    country: Optional[str]
    sha256sum: str = None
    ssdeep: str = None
    date: datetime 
    chunk: bool = False


class RansomwareHit(BaseModel):
    page: RansomwarePage


class Hit(BaseModel):

    page: Page
    tag: List[Tag]
    email: List[Email] = Field(default_factory=lambda: [])
    paste: List[Paste] = Field(default_factory=lambda: [])
    skype: List[Skype] = Field(default_factory=lambda: [])
    telegram: List[Telegram] = Field(default_factory=lambda: [])
    whatsapp: List[WhatsApp] = Field(default_factory=lambda: [])
    bitcoin_address: List[BitcoinAddress] = Field(default_factory=lambda: [])
    polkadot_address: List[PolkadotAddress] = Field(default_factory=lambda: [])
    ethereum_address: List[EthereumAddress] = Field(default_factory=lambda: [])
    monero_address: List[MoneroAddress] = Field(default_factory=lambda: [])
    ripple_address: List[RippleAddress] = Field(default_factory=lambda: [])
    zcash_address: List[ZcashAddress] = Field(default_factory=lambda: [])


class RansomFeedHit(BaseModel):

    id: str
    company: Optional[str]
    company_link: Optional[str]
    link: str
    group: RansomGroup
    date: datetime
    info: Optional[str]
    country: Optional[str]


class TelegramFeedHit(BaseModel):

    id: str
    telegram: List[str]
    date: datetime
    url: str
    path: str
    network: str


class Result(BaseModel):

    # TODO Añadir paginación, query, etc?
    total: int = 0
    hits: Union[List[Hit], List[RansomFeedHit], List[TelegramFeedHit], List[RansomwareHit]] = Field(
        default_factory=lambda: [])
        
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if self.total <= 0:
            self.total = len(self.hits)

    def get_type(self) -> type:

        if len(self.hits) <= 0:
            return NoneType

        return type(self.hits[0])


# TODO Move API responses to other class
class VysionResponse(BaseModel):
    """
    VysionResponse is a json:api flavoured response from the API
    """

    data: Result  # TODO Add type to all JSON:API responses


class VysionError(BaseModel):

    class StatusCode(int, Enum):

        UNK = 000
        OK = 200
        REQ_ERROR = 400
        UNAUTHORIZED = 403
        NOT_FOUND = 404
        INTERNAL_ERROR = 500

    code: StatusCode = StatusCode.UNK
    message: str = "UNK_ERR"



#!/usr/bin/env python3
"""
Copyright 2022 Byron Labs S.L.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import IntEnum


class Services(IntEnum):

    acr_nema = 104
    afpovertcp = 548
    afs3_bos = 7007
    afs3_callback = 7001
    afs3_fileserver = 7000
    afs3_kaserver = 7004
    afs3_prserver = 7002
    afs3_rmtsys = 7009
    afs3_update = 7008
    afs3_vlserver = 7003
    afs3_volser = 7005
    amanda = 10080
    amandaidx = 10082
    amidxtape = 10083
    amqp = 5672
    amqps = 5671
    asf_rmcp = 623
    asp = 27374
    auth = 113
    babel = 6696
    bacula_dir = 9101
    bacula_fd = 9102
    bacula_sd = 9103
    bbs = 7000
    bgp = 179
    bgpd = 2605
    biff = 512
    binkp = 24554
    bootpc = 68
    bootps = 67
    canna = 5680
    cfengine = 5308
    chargen = 19
    cisco_sccp = 2000
    clc_build_daemon = 8990
    clearcase = 371
    cmip_agent = 164
    cmip_man = 163
    codaauth2 = 370
    codasrv = 2432
    codasrv_se = 2433
    csync2 = 30865
    cvspserver = 2401
    daap = 3689
    datametrics = 1645
    daytime = 13
    db_lsp = 17500
    dcap = 22125
    dhcpv6_client = 546
    dhcpv6_server = 547
    dicom = 11112
    dict = 2628
    dircproxy = 57000
    discard = 9
    distcc = 3632
    domain = 53
    domain_s = 853
    echo = 7
    epmap = 135
    epmd = 4369
    exec = 512
    f5_globalsite = 2792
    f5_iquery = 4353
    fax = 4557
    fido = 60179
    finger = 79
    font_service = 7100
    freeciv = 5556
    fsp = 21
    ftp = 21
    ftp_data = 20
    ftps = 990
    ftps_data = 989
    gdomap = 538
    gds_db = 3050
    git = 9418
    gnunet = 2086
    gnutella_rtr = 6347
    gnutella_svc = 6346
    gopher = 70
    gpsd = 2947
    gris = 2135
    groupwise = 1677
    gsidcap = 22128
    gsiftp = 2811
    gsigatekeeper = 2119
    hkp = 11371
    http = 80
    http_alt = 8080
    https = 443
    hylafax = 4559
    iax = 4569
    icpv2 = 3130
    imap2 = 143
    imaps = 993
    ingreslock = 1524
    ipp = 631
    iprop = 2121
    ipsec_nat_t = 4500
    ipx = 213
    ircd = 6667
    ircs_u = 6697
    isakmp = 500
    iscsi_target = 3260
    isisd = 2608
    isns = 3205
    iso_tsap = 102
    kamanda = 10081
    kerberos = 88
    kerberos4 = 750
    kerberos_adm = 749
    kerberos_master = 751
    kermit = 1649
    klogin = 543
    kpasswd = 464
    krb_prop = 754
    kshell = 544
    l2f = 1701
    ldap = 389
    ldaps = 636
    ldp = 646
    login = 513
    lotusnote = 1352
    mailq = 174
    mdns = 5353
    microsoft_ds = 445
    moira_db = 775
    moira_update = 777
    moira_ureg = 779
    mon = 2583
    ms_sql_m = 1434
    ms_sql_s = 1433
    ms_wbt_server = 3389
    mtn = 4691
    munin = 4949
    mysql = 3306
    mysql_proxy = 6446
    nbd = 10809
    nbp = 2
    netbios_dgm = 138
    netbios_ns = 137
    netbios_ssn = 139
    netstat = 15
    nfs = 2049
    nntp = 119
    nntps = 563
    nqs = 607
    nrpe = 5666
    nsca = 5667
    ntalk = 518
    ntp = 123
    ntske = 4460
    nut = 3493
    omniorb = 8088
    openvpn = 1194
    ospf6d = 2606
    ospfapi = 2607
    ospfd = 2604
    passwd_server = 752
    pawserv = 345
    pop3 = 110
    pop3s = 995
    poppassd = 106
    postgresql = 5432
    predict = 1210
    printer = 515
    proofd = 1093
    ptp_event = 319
    ptp_general = 320
    puppet = 8140
    qmqp = 628
    qmtp = 209
    qotd = 17
    radius = 1812
    radius_acct = 1813
    radmin_port = 4899
    redis = 6379
    remctl = 4373
    ripd = 2602
    ripngd = 2603
    rmiregistry = 1099
    rmtcfg = 1236
    rootd = 1094
    route = 520
    rpc2portmap = 369
    rplay = 5555
    rsync = 873
    rtcm_sc104 = 2101
    rtmp = 1
    rtsp = 554
    saft = 487
    sa_msg_port = 1646
    sane_port = 6566
    sge_execd = 6445
    sge_qmaster = 6444
    sgi_cad = 17004
    sgi_cmsd = 17001
    sgi_crsd = 17002
    sgi_gcd = 17003
    shell = 514
    sieve = 4190
    silc = 706
    sip = 5060
    sip_tls = 5061
    skkserv = 1178
    smtp = 25
    smux = 199
    snmp = 161
    snmp_trap = 162
    snpp = 444
    socks = 1080
    spamd = 783
    ssh = 22
    submission = 587
    submissions = 465
    sunrpc = 111
    supfiledbg = 1127
    supfilesrv = 871
    suucp = 4031
    svn = 3690
    svrloc = 427
    syslog = 514
    syslog_tls = 6514
    sysrqd = 4094
    systat = 11
    tacacs = 49
    talk = 517
    tcpmux = 1
    telnet = 23
    telnets = 992
    tfido = 60177
    tftp = 69
    time = 37
    tinc = 655
    tproxy = 8081
    uucp = 540
    venus = 2430
    venus_se = 2431
    webmin = 10000
    who = 513
    whois = 43
    wnn6 = 22273
    x11 = 6000
    x11_1 = 6001
    x11_2 = 6002
    x11_3 = 6003
    x11_4 = 6004
    x11_5 = 6005
    x11_6 = 6006
    x11_7 = 6007
    xdmcp = 177
    xinetd = 9098
    xmms2 = 9667
    xmpp_client = 5222
    xmpp_server = 5269
    xtel = 1313
    xtelw = 1314
    z3950 = 210
    zabbix_agent = 10050
    zabbix_trapper = 10051
    zebra = 2601
    zebrasrv = 2600
    zephyr_clt = 2103
    zephyr_hm = 2104
    zephyr_srv = 2102
    zip = 6
    zope = 9673
    zope_ftp = 8021
    zserv = 346
